package cn.com.qmth.examcloud.service.core.service;

import cn.com.qmth.examcloud.service.core.dao.Org;
import cn.com.qmth.examcloud.service.core.dao.Student;
import cn.com.qmth.examcloud.service.core.dto.UserInfo;
import cn.com.qmth.examcloud.service.core.enums.UserType;
import cn.com.qmth.examcloud.service.core.params.UserParam;
import cn.com.qmth.examcloud.service.core.repo.OrgRepo;
import cn.com.qmth.examcloud.service.core.repo.StudentRepo;
import cn.com.qmth.examcloud.service.core.repo.UserRepo;
import cn.com.qmth.examcloud.service.core.dao.User;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.startsWith;

/**
 * 用户服务类
 * Created by songyue on 17/1/13.
 */
@Service
public class UserService {

    @Autowired
    UserRepo userRepo;

    @Autowired
    StudentRepo studentRepo;

    @Autowired
    OrgRepo orgRepo;

    /**
     * 初始化密码
     * @param userId
     */
    public void initPassword(long userId){
        User user = userRepo.findById(userId);
        //根据用户类型采用不同的初始化策略
        if(user.getType() == UserType.NOT_STUDENT){
            //初始化为默认密码
            user.setPassword(UserParam.DEFAULT_PASSWORD);

        }else if(user.getType() == UserType.STUDENT){
            //截取身份证后6位为学生登录密码
            Student student = studentRepo.findByUserId(userId);
            user.setPassword(StringUtils.substring(student.getIdentityNumber(),-6,-1));
        }
        userRepo.save(user);
    }


    /**
     * 一般登录
     * @param loginName
     * @param password
     * @return
     */
    public ResponseEntity login(String loginName,String password){
        User user = userRepo.findByLoginName(loginName);
        return loginProcess(user,password);

    }

    /**
     * 二级登录
     * @param orgId
     * @param loginName
     * @param password
     * @return
     */
    public ResponseEntity login(long orgId,String loginName,String password){
        User user = userRepo.findByRootOrgIdAndLoginName(orgId,loginName);
        return loginProcess(user,password);
    }

    /**
     * 登录处理
     * @param user
     * @param password
     * @return
     */
    public ResponseEntity loginProcess(User user,String password){
        if(user == null){
            return new ResponseEntity("该用户不存在",HttpStatus.NOT_FOUND);
        }else if(!user.getPassword().equals(password)){
            return new ResponseEntity("密码错误",HttpStatus.EXPECTATION_FAILED);
        }else{
            String token = "";
            return new ResponseEntity(getUserInfo(user,token),HttpStatus.OK);
        }

    }

    /**
     * 获取用户信息DTO
     * @param user
     * @param token
     * @return
     */
    public UserInfo getUserInfo(User user,String token){
        UserInfo userInfo = new UserInfo();
        Org org = orgRepo.findById(user.getOrgId());
        Org rootOrg = orgRepo.findById(user.getRootOrgId());
        userInfo.setUserId(user.getId());
        userInfo.setName(user.getName());
        userInfo.setAvatar(user.getAvatar());
        userInfo.setLoginName(user.getLoginName());
        userInfo.setOrgName(org.getName());
        userInfo.setRootOrgLogo(rootOrg.getLogo());
        userInfo.setRootOrgName(rootOrg.getName());
        userInfo.setToken(token);
        userInfo.setType(user.getType().toString());
        return userInfo;
    }

}
